@@ -35,11 +35,28 @@
 **  RCS:
 **
 **      $Source: /afs/pitt.edu/usr12/dgm/work/IMAP_Proxy/src/RCS/config.c,v $
-**      $Id: config.c,v 1.8 2003/10/23 06:18:58 dgm Exp $
+**      $Id: config.c,v 1.13 2005/01/12 17:49:51 dgm Exp $
 **      
 **  Modification History:
 **
 **      $Log: config.c,v $
+**      Revision 1.13  2005/01/12 17:49:51  dgm
+**      Applied patch by David Lancaster to provide force_tls config
+**      option.
+**
+**      Revision 1.12  2004/11/10 15:26:25  dgm
+**      Explictly NULL terminate all strings that are the result of an strncpy.
+**
+**      Revision 1.11  2004/10/11 18:01:29  dgm
+**      Added foreground mode configuration option.
+**
+**      Revision 1.10  2004/02/24 14:57:47  dgm
+**      Added new config option 'enable_select_cache'.
+**
+**      Revision 1.9  2003/11/14 15:03:58  dgm
+**      Patch by Geoffrey Hort <g.hort@unsw.edu.au> to allow
+**      configurable listen address.
+**
 **      Revision 1.8  2003/10/23 06:18:58  dgm
 **      Fixed bug in SetBooleanValue doing upcase of Value.
 **
@@ -119,6 +136,7 @@ struct Config_Struct ConfigTable[ 100 ];
  */
 #define ADD_TO_TABLE( KEYWORD, SETFUNCTION, STA, INDEX ) \
         strncpy( ConfigTable[ INDEX ].Keyword, KEYWORD, MAX_KEYWORD_LEN -1 ); \
+        ConfigTable[ INDEX ].Keyword[ MAX_KEYWORD_LEN - 1 ] = '\0'; \
         ConfigTable[ INDEX ].SetFunction = SETFUNCTION; \
         ConfigTable[ INDEX ].StorageAddress = STA; \
         INDEX++;
@@ -338,6 +356,9 @@ extern void SetConfigOptions( char *ConfigFile )
     ADD_TO_TABLE( "listen_port", SetNumericValue, 
 		  &PC_Struct.listen_port, index );
 
+    ADD_TO_TABLE( "listen_address", SetStringValue,
+		  &PC_Struct.listen_addr, index );
+
     ADD_TO_TABLE( "server_port", SetNumericValue, 
 		  &PC_Struct.server_port, index );
 
@@ -380,6 +401,15 @@ extern void SetConfigOptions( char *ConfigFile )
     ADD_TO_TABLE( "send_tcp_keepalives", SetBooleanValue,
 		  &PC_Struct.send_tcp_keepalives, index );
 
+    ADD_TO_TABLE( "enable_select_cache", SetBooleanValue,
+		  &PC_Struct.enable_select_cache, index );
+
+    ADD_TO_TABLE( "foreground_mode", SetBooleanValue,
+		  &PC_Struct.foreground_mode, index );
+
+    ADD_TO_TABLE( "force_tls", SetBooleanValue,
+		  &PC_Struct.force_tls, index );
+    
     ConfigTable[index].Keyword[0] = '\0';
     
     FP = fopen( ConfigFile, "r" );
