@@ -39,11 +39,19 @@
 **  RCS:
 **
 **	$Source: /afs/pitt.edu/usr12/dgm/work/IMAP_Proxy/src/RCS/request.c,v $
-**	$Id: request.c,v 1.17 2003/10/09 14:11:13 dgm Exp $
+**	$Id: request.c,v 1.19 2004/11/10 15:33:04 dgm Exp $
 **      
 **  Modification History:
 **
 **	$Log: request.c,v $
+**	Revision 1.19  2004/11/10 15:33:04  dgm
+**	Explictly NULL terminate all strings that are the result
+**	of strncpy.  Also enforce checking of LiteralBytesRemaining
+**	after any call to IMAP_Line_Read.
+**
+**	Revision 1.18  2004/02/24 15:19:06  dgm
+**	Added support for SELECT caching.
+**
 **	Revision 1.17  2003/10/09 14:11:13  dgm
 **	bugfix: set TotalClientLogins to zero in cmd_resetcounters, submitted
 **	by Geoffrey Hort <g.hort@unsw.edu.au>.  Changes to allow syslogging of the
@@ -166,7 +174,7 @@ static int cmd_trace( ITD_Struct *, char *, char * );
 static int cmd_dumpicc( ITD_Struct *, char * );
 static int cmd_newlog( ITD_Struct *, char * );
 static int cmd_resetcounters( ITD_Struct *, char * );
-static int Raw_Proxy( ITD_Struct *, ITD_Struct * );
+static int Raw_Proxy( ITD_Struct *, ITD_Struct *, ISC_Struct * );
 
 
 
@@ -430,6 +438,7 @@ static int cmd_trace( ITD_Struct *itd, char *Tag, char *Username )
     }
     
     strncpy( TraceUser, Username, sizeof TraceUser - 1 );
+    TraceUser[ sizeof TraceUser - 1 ] = '\0';
     
     snprintf( SendBuf, BufLen, "%s OK Tracing enabled\r\n", Tag );
     if ( IMAP_Write( itd->conn, SendBuf, strlen(SendBuf) ) == -1 )
@@ -615,6 +624,15 @@ static int cmd_authenticate_login( ITD_Struct *Client,
 	syslog( LOG_NOTICE, "%s: Failed to read base64 encoded username from client on socket %d", fn, Client->conn->sd );
 	return( -1 );
     }
+
+    /*
+     * Don't accept literals from the client here.
+     */
+    if ( Client->LiteralBytesRemaining )
+    {
+	syslog( LOG_NOTICE, "%s: Read unexpected literal specifier from client on socket %d", fn, Client->conn->sd );
+	return( -1 );
+    }
     
     /*
      * Easy, but not perfect sanity check.  If the client sent enough data
@@ -652,6 +670,12 @@ static int cmd_authenticate_login( ITD_Struct *Client,
     }
 
     BytesRead = IMAP_Line_Read( Client );
+
+    if ( Client->LiteralBytesRemaining )
+    {
+	syslog( LOG_ERR, "%s: received unexpected literal specifier from client on socket %d", fn, Client->conn->sd );
+	return( -1 );
+    }
     
     if ( BytesRead == -1 )
     {
@@ -740,7 +764,7 @@ static int cmd_authenticate_login( ITD_Struct *Client,
     }
     UnLockMutex( &trace );
 
-    rc = Raw_Proxy( Client, &Server );
+    rc = Raw_Proxy( Client, &Server, &Server.conn->ISC );
     
     Client->TraceOn = 0;
     Server.TraceOn = 0;
@@ -879,7 +903,7 @@ static int cmd_login( ITD_Struct *Client,
     }
     UnLockMutex( &trace );
 
-    rc = Raw_Proxy( Client, &Server );
+    rc = Raw_Proxy( Client, &Server, &Server.conn->ISC );
 
     /*
      * It's not necessary to take out the trace mutex here.  The reason
@@ -915,7 +939,8 @@ static int cmd_login( ITD_Struct *Client,
  * Authors:	Dave McMurtrie <davemcmurtrie@hotmail.com>
  *--
  */
-static int Raw_Proxy( ITD_Struct *Client, ITD_Struct *Server )
+static int Raw_Proxy( ITD_Struct *Client, ITD_Struct *Server,
+		      ISC_Struct *ISC )
 {
     char *fn = "Raw_Proxy()";
     struct pollfd fds[2];
@@ -926,6 +951,7 @@ static int Raw_Proxy( ITD_Struct *Client, ITD_Struct *Server )
     char *CP;
     char TraceBuf[ BUFSIZE ];
     char SendBuf[ BUFSIZE ];
+    int rc;
     
 #define SERVER 0
 #define CLIENT 1
@@ -1108,9 +1134,11 @@ static int Raw_Proxy( ITD_Struct *Client, ITD_Struct *Server )
 		}
 		
 	    
-		/* this is a command -- is it logout? */
-		CP = memchr(Client->ReadBuf, ' ',
-			    Client->ReadBytesProcessed );
+		/* 
+		 * This is a command.  What command is it?
+		 */
+		CP = memchr( Client->ReadBuf, ' ',
+			     Client->ReadBytesProcessed );
 	    
 		if ( CP )
 		{
@@ -1122,61 +1150,101 @@ static int Raw_Proxy( ITD_Struct *Client, ITD_Struct *Server )
 			 * Since we want to potentially reuse this server
 			 * connection, we want to return it to an unselected 
 			 * state.  Use UNSELECT if the server supports it.
-			 * Otherwise, EXAMINE a null mailbox.
+			 * Otherwise, EXAMINE a null mailbox.  If SELECT
+			 * caching is enabled, don't do this.
 			 */
-			snprintf( SendBuf, sizeof SendBuf - 1,
-				  "C64 %s\r\n", ( (PC_Struct.support_unselect) ? "UNSELECT" : "EXAMINE \"\"" ) );
-			
-			IMAP_Write( Server->conn, SendBuf,
-				    strlen(SendBuf) );
-			/*
-			 * To be more correct, we should send any untagged
-			 * data back to the client before we're done.
-			 */
-			for( ;; )
+			if ( ! PC_Struct.enable_select_cache )
 			{
-			    /*
-			     * If the server wants to send a literal for
-			     * some reason, bag it...
-			     */
-			    if ( Server->LiteralBytesRemaining )
-				break;
-			    
-			    status = IMAP_Line_Read( Server );
+			    snprintf( SendBuf, sizeof SendBuf - 1,
+				      "C64 %s\r\n", ( (PC_Struct.support_unselect) ? "UNSELECT" : "EXAMINE \"\"" ) );
 			    
+			    IMAP_Write( Server->conn, SendBuf,
+					strlen(SendBuf) );
 			    /*
-			     * If there's an error reading from the server,
-			     * we'll catch it when (if) we try to reuse this
-			     * connection.
+			     * To be more correct, we should send any untagged
+			     * data back to the client before we're done.
 			     */
-			    if ( ( status == -1 ) || ( status == 0 ) )
-				break;
-			    
-			    /*
-			     * If it's not untagged data, we're done.
-			     */
-			    if ( Server->ReadBuf[0] != '*' )
-				break;
-
-			    BytesSent = IMAP_Write( Client->conn, 
-						    Server->ReadBuf, status );
-			    if ( BytesSent == -1 )
+			    for( ;; )
 			    {
-				syslog( LOG_ERR, "%s: IMAP_Write() failed sending data to client on sd [%d]: %s", fn, Client->conn->sd, strerror( errno ) );
-			    }
+				/*
+				 * If the server wants to send a literal for
+				 * some reason, bag it...
+				 */
+				if ( Server->LiteralBytesRemaining )
+				    break;
+				
+				status = IMAP_Line_Read( Server );
+				
+				/*
+				 * If there's an error reading from the server,
+				 * we'll catch it when (if) we try to reuse this
+				 * connection.
+				 */
+				if ( ( status == -1 ) || ( status == 0 ) )
+				    break;
 			    
-			}
+				/*
+				 * If it's not untagged data, we're done.
+				 */
+				if ( Server->ReadBuf[0] != '*' )
+				    break;
+				
+				BytesSent = IMAP_Write( Client->conn, 
+							Server->ReadBuf, status );
+				if ( BytesSent == -1 )
+				{
+				    syslog( LOG_ERR, "%s: IMAP_Write() failed sending data to client on sd [%d]: %s", fn, Client->conn->sd, strerror( errno ) );
+				}
+			    }
+			} /* if ( ! PC_Struct.enable_select_cache ) */
 
 			memset( Server->ReadBuf, 0, sizeof Server->ReadBuf );
 			
 			return( 1 );
 		    }
-		}
 		
-		/*
-		 * it's some command other than a LOGOUT...
-		 * just ship it over
-		 */
+		    /*
+		     * it's some command other than a LOGOUT...
+		     * If we care about SELECT caching, do that now.
+		     */
+		    if ( PC_Struct.enable_select_cache )
+		    {
+			if ( !strncasecmp( CP, "SELECT ", 7 ) )
+			{
+			    rc = Handle_Select_Command( Client, Server,
+							ISC, Client->ReadBuf,
+							status );
+			    
+			    if ( rc == 0 )
+				continue;
+
+			    if ( rc == -1 )
+				return( -1 );
+
+			    /* 
+			     * if Handle_Select_Command() returned 1,
+			     * fall through the rest of the logic and the
+			     * SELECT command should be proxied without
+			     * looking at the cache.
+			     */
+			    
+			} /* if the command is SELECT */
+
+			/*
+			 * SELECT caching is enabled and we've encountered
+			 * a command other than SELECT.  See if we should
+			 * invalidate the SELECT cache or not.
+			 */
+			if ( ! Is_Safe_Command( CP ) )
+			{
+			    Invalidate_Cache_Entry( ISC );
+			}
+			
+		    } /* if ( PC_Struct.enable_select_cache ) */
+		    
+		} /* if ( CP ) */
+		
+
 		for ( ; ; )
 		{
 		    BytesSent = IMAP_Write( Server->conn, Client->ReadBuf, status );
@@ -1185,7 +1253,7 @@ static int Raw_Proxy( ITD_Struct *Client, ITD_Struct *Server )
 			if ( errno == EINTR )
 			    continue;
 			
-			syslog(LOG_ERR, "%s: IMAP_Write() failed sending data to client on sd [%d]: %s", fn, Client->conn->sd, strerror( errno ) );
+			syslog(LOG_ERR, "%s: IMAP_Write() failed sending data to server on sd [%d]: %s", fn, Server->conn->sd, strerror( errno ) );
 			return( -1 );
 		    }
 		    break;
@@ -1427,7 +1495,7 @@ extern void HandleRequest( int clientsd )
 	PollFailCount = 0;
 
 	BytesRead = IMAP_Line_Read( &Client );
-	
+
 	if ( BytesRead == -1 )
 	{
 	    IMAPCount->CurrentClientConnections--;
@@ -1483,18 +1551,42 @@ extern void HandleRequest( int clientsd )
 	 * appropriate...
 	 */
 	strncpy( S_Tag, Tag, MAXTAGLEN - 1 );
+	S_Tag[ MAXTAGLEN - 1 ] = '\0';
+	
 	if ( ! strcasecmp( (const char *)Command, "NOOP" ) )
 	{
+	    if ( Client.LiteralBytesRemaining )
+	    {
+		syslog( LOG_ERR, "%s: Unexpected literal specifier read from client on socket %d as part of NOOP command -- disconnecting client", fn, Client.conn->sd );
+		IMAPCount->CurrentClientConnections--;
+		close( Client.conn->sd );
+		return;
+	    }
+	    
 	    cmd_noop( &Client, S_Tag );
 	    continue;
 	}
 	else if ( ! strcasecmp( (const char *)Command, "CAPABILITY" ) )
 	{
+	    if ( Client.LiteralBytesRemaining )
+	    {
+		syslog( LOG_ERR, "%s: Unexpected literal specifier read from client on socket %d as part of CAPABILITY command -- disconnecting client", fn, Client.conn->sd );
+		IMAPCount->CurrentClientConnections--;
+		close( Client.conn->sd );
+		return;
+	    }
 	    cmd_capability( &Client, S_Tag );
 	    continue;
 	}
 	else if ( ! strcasecmp( (const char *)Command, "AUTHENTICATE" ) )
 	{
+	    if ( Client.LiteralBytesRemaining )
+	    {
+		syslog( LOG_ERR, "%s: Unexpected literal specifier read from client on socket %d as part of AUTHENTICATE command -- disconnecting client", fn, Client.conn->sd );
+		IMAPCount->CurrentClientConnections--;
+		close( Client.conn->sd );
+		return;
+	    }
 	    AuthMech = memtok( NULL, EndOfLine, &Lasts );
 	    if ( !AuthMech )
 	    {
@@ -1507,7 +1599,7 @@ extern void HandleRequest( int clientsd )
 		}
 		continue;
 	    }
-
+	    
 	    if ( !strcasecmp( (const char *)AuthMech, "LOGIN" ) )
 	    {
 		rc = cmd_authenticate_login( &Client, S_Tag );
@@ -1522,6 +1614,7 @@ extern void HandleRequest( int clientsd )
 		    if ( Tag )
 		    {
 			strncpy( S_Tag, Tag, MAXTAGLEN - 1 );
+			S_Tag[ MAXTAGLEN - 1 ] = '\0';
 			cmd_logout( &Client, S_Tag );
 		    }
 		}
@@ -1544,10 +1637,17 @@ extern void HandleRequest( int clientsd )
 		}
 		continue;
 	    }
-
+	    
 	}
 	else if ( ! strcasecmp( (const char *)Command, "LOGOUT" ) )
 	{
+	    if ( Client.LiteralBytesRemaining )
+	    {
+		syslog( LOG_ERR, "%s: Unexpected literal specifier read from client on socket %d as part of LOGOUT command -- disconnecting client", fn, Client.conn->sd );
+		IMAPCount->CurrentClientConnections--;
+		close( Client.conn->sd );
+		return;
+	    }
 	    cmd_logout( &Client, S_Tag );
 	    IMAPCount->CurrentClientConnections--;
 	    close( Client.conn->sd );
@@ -1555,22 +1655,50 @@ extern void HandleRequest( int clientsd )
 	}
 	else if ( ! strcasecmp( (const char *)Command, "P_TRACE" ) )
 	{
+	    if ( Client.LiteralBytesRemaining )
+	    {
+		syslog( LOG_ERR, "%s: Unexpected literal specifier read from client on socket %d as part of P_TRACE command -- disconnecting client", fn, Client.conn->sd );
+		IMAPCount->CurrentClientConnections--;
+		close( Client.conn->sd );
+		return;
+	    }
 	    Username = memtok( NULL, EndOfLine, &Lasts );
 	    cmd_trace( &Client, S_Tag, Username );
 	    continue;
 	}
 	else if ( ! strcasecmp( (const char *)Command, "P_DUMPICC" ) )
 	{
+	    if ( Client.LiteralBytesRemaining )
+	    {
+		syslog( LOG_ERR, "%s: Unexpected literal specifier read from client on socket %d as part of P_DUMPICC command -- disconnecting client", fn, Client.conn->sd );
+		IMAPCount->CurrentClientConnections--;
+		close( Client.conn->sd );
+		return;
+	    }
 	    cmd_dumpicc( &Client, S_Tag );
 	    continue;
 	}
 	else if ( ! strcasecmp( (const char *)Command, "P_RESETCOUNTERS" ) )
 	{
+	    if ( Client.LiteralBytesRemaining )
+	    {
+		syslog( LOG_ERR, "%s: Unexpected literal specifier read from client on socket %d as part of P_RESETCOUNTERS command -- disconnecting client", fn, Client.conn->sd );
+		IMAPCount->CurrentClientConnections--;
+		close( Client.conn->sd );
+		return;
+	    }
 	    cmd_resetcounters( &Client, S_Tag );
 	    continue;
 	}
 	else if ( ! strcasecmp( (const char *)Command, "P_NEWLOG" ) )
 	{
+	    if ( Client.LiteralBytesRemaining )
+	    {
+		syslog( LOG_ERR, "%s: Unexpected literal specifier read from client on socket %d as part of P_NEWLOG command -- disconnecting client", fn, Client.conn->sd );
+		IMAPCount->CurrentClientConnections--;
+		close( Client.conn->sd );
+		return;
+	    }
 	    cmd_newlog( &Client, S_Tag );
 	    continue;
 	}
@@ -1594,7 +1722,8 @@ extern void HandleRequest( int clientsd )
 		}
 		continue;
 	    }
-	    strncpy( S_UserName, Username, sizeof S_UserName - 1 );	    
+	    strncpy( S_UserName, Username, sizeof S_UserName - 1 );
+	    S_UserName[ sizeof S_UserName - 1 ] = '\0';
 	    
 	    /*
 	     * Clients can send the password as a literal bytestream.  Check
@@ -1701,6 +1830,7 @@ extern void HandleRequest( int clientsd )
 		
 		*CP = '\0';
 		strncpy( S_Password, Lasts, sizeof S_Password - 1 );
+		S_Password[ sizeof S_Password - 1 ] = '\0';
 	    }
 	    
 
@@ -1732,6 +1862,7 @@ extern void HandleRequest( int clientsd )
 		if ( Tag )
 		{
 		    strncpy( S_Tag, Tag, MAXTAGLEN - 1 );
+		    S_Tag[ MAXTAGLEN - 1 ] = '\0';
 		    cmd_logout( &Client, S_Tag );
 		}
 	    }
@@ -1751,6 +1882,14 @@ extern void HandleRequest( int clientsd )
 	     * same way the cyrus implementation does -- tell the client to
 	     * log in first.
 	     */
+	    if ( Client.LiteralBytesRemaining )
+	    {
+		syslog( LOG_ERR, "%s: Unexpected literal specifier read from client on socket %d as part of unknown command -- disconnecting client", fn, Client.conn->sd );
+		IMAPCount->CurrentClientConnections--;
+		close( Client.conn->sd );
+		return;
+	    }
+	    
 	    snprintf( SendBuf, BufLen, "%s BAD Please login first\r\n", Tag, Command );
 	    if ( IMAP_Write( Client.conn, SendBuf, strlen(SendBuf) ) == -1 )
 	    {
